//
//  EditEnhance_f.shader
//  BeautyPlus
//
//  Created by Webster Wu on 2/18/16.
//  Copyright © 2016 美图网. All rights reserved.
//

precision highp float;

varying vec2 texcoordOut;

uniform sampler2D inputImageTexture0;

uniform float alpha;

// RGB 2 Gray conversion factors.
const float kLuminanceConversionRedFactor = 0.299;
const float kLuminanceConversionGreenFactor = 0.587;
const float kLuminanceConversionBlueFactor = 0.114;

//饱和度处理
vec3 ApplySaturation(vec3 color) {
    // If the saturation is to be increased, reduce the effect in case the current
    // pixel is already quite saturated.
    float saturation = alpha;
    if (alpha > 0.){
        float r = color.r;
        float g = color.g;
        float b = color.b;
        float maxValue = max(r, max(g, b));
        float minValue = min(r, min(g, b));
        
        float current_saturation = 0.;
        if (maxValue != 0.) {
            current_saturation = (maxValue - minValue) / maxValue;
        }
        
        float falloff = 1. - current_saturation;
        saturation *= falloff * 2.;
    }
    saturation += 1.;
    
    float luminance = kLuminanceConversionRedFactor * color.r +
    kLuminanceConversionGreenFactor * color.g +
    kLuminanceConversionBlueFactor * color.b;
    
    // By design, the saturation algorithm may produce values outside of the
    // range [0, 1].
    return clamp(mix(vec3(luminance), color, saturation), 0.0, 1.0);
}

void main()
{
    vec4 oralColor = texture2D(inputImageTexture0, texcoordOut);
    
    gl_FragColor = vec4(ApplySaturation(oralColor.rgb), 1.0);
}
