//
//  EditEnhance_f.shader
//  BeautyPlus
//
//  Created by Webster Wu on 2/18/16.
//  Copyright © 2016 美图网. All rights reserved.
//

precision highp float;

uniform sampler2D inputImageTexture0;

varying highp vec2 texcoordOut;
uniform lowp vec3 redShift;

uniform lowp vec3 orangeShift;
uniform lowp vec3 yellowShift;
uniform lowp vec3 greenShift;
uniform lowp vec3 aquaShift;
uniform lowp vec3 blueShift;
uniform lowp vec3 purpleShift;
uniform lowp vec3 magentaShift;

const lowp float redOrange = 22.0 / 360.0;
const lowp float orangeYellow = 38.0 / 360.0;
const lowp float yellowGreen = 56.0 / 360.0;
const lowp float greenAqua = 161.0 / 360.0;
const lowp float aquaBlue = 174.0 / 360.0;
const lowp float bluePurple = 285.0 / 360.0;
const lowp float purlpeMagenta = 297.0 / 360.0;
const lowp float magentaRed = 328.0 / 360.0;
const highp vec3 labWhiteColor = vec3(100.0,0.0052604999583039,-0.0104081845252679);

highp vec3 HUEtoRGB(highp float H)
{
    highp float R = abs(H * 6.0 - 3.0) - 1.0;
    highp float G = 2.0 - abs(H * 6.0 - 2.0);
    highp float B = 2.0 - abs(H * 6.0 - 4.0);
    return clamp(vec3(R,G,B), 0.0, 1.0);
}

highp float Epsilon = 1e-10;

highp vec3 RGBtoHCV(highp vec3 RGB)
{
    highp vec4 P = (RGB.g < RGB.b) ? vec4(RGB.bg, -1.0, 2.0/3.0) : vec4(RGB.gb, 0.0, -1.0/3.0);
    highp vec4 Q = (RGB.r < P.x) ? vec4(P.xyw, RGB.r) : vec4(RGB.r, P.yzx);
    highp float C = Q.x - min(Q.w, Q.y);
    highp float H = abs((Q.w - Q.y) / (6.0 * C + Epsilon) + Q.z);
    return vec3(H, C, Q.x);
}

highp vec3 RGBtoHSL(highp vec3 RGB)
{
    highp vec3 HCV = RGBtoHCV(RGB);
    highp float L = HCV.z - HCV.y * 0.5;
    highp float S = HCV.y / (1.0 - abs(L * 2.0 - 1.0) + Epsilon);
    return vec3(HCV.x, S, L);
}

highp vec3 HSLtoRGB(highp vec3 HSL)
{
    highp vec3 RGB = HUEtoRGB(HSL.x);
    highp float C = (1.0 - abs(2.0 * HSL.z - 1.0)) * HSL.y;
    return (RGB - 0.5) * C + HSL.z;
}

highp vec3 hsl2rgb(highp vec3 c)
{
    c = vec3(fract(c.x), clamp(c.yz, 0.0, 1.0));
    highp vec3 rgb = clamp(abs(mod(c.x * 6.0 + vec3(0.0, 4.0, 2.0), 6.0) - 3.0) - 1.0, 0.0, 1.0);
    return c.z + c.y * (rgb - 0.5) * (1.0 - abs(2.0 * c.z - 1.0));
}

highp vec3 RGB2Lab(highp vec3 rgb)
{
    highp float R = rgb.x;
    highp float G = rgb.y;
    highp float B = rgb.z;
    highp float T = 0.008856;
    highp float X = R * 0.412453 + G * 0.357580 + B * 0.180423;
    highp float Y = R * 0.212671 + G * 0.715160 + B * 0.072169;
    highp float Z = R * 0.019334 + G * 0.119193 + B * 0.950227;
    X = X / 0.950456;
    Y = Y;
    Z = Z / 1.088754;
    int XT = 0;
    int YT = 0;
    int ZT = 0;
    if(X > T){ XT = 1;
    } if(Y > T){ YT = 1;
    } if(Z > T){ ZT = 1;
    } highp float Y3 = pow(Y,1.0/3.0);
    highp float fX;
    highp float fY;
    highp float fZ;
    if(XT!=0){ fX = pow(X, 1.0/3.0);
    } else{ fX = 7.787 * X + 16.0/116.0;
    } if(YT!=0){ fY = Y3;
    } else{ fY = 7.787 * Y + 16.0/116.0 ;
    } if(ZT!=0){ fZ = pow(Z,1.0/3.0);
    } else{ fZ = 7.787 * Z + 16.0/116.0;
    } highp float L;
    if(YT!=0){ L = (116.0 * Y3) - 16.0;
    }else { L = 903.3 * Y;
    } highp float a = 500.0 * ( fX - fY );
    highp float b = 200.0 * ( fY - fZ );
    return vec3(L,a,b);
}

highp vec3 smoothTreatment(highp vec3 hsv, highp float hueEdge0, highp float hueEdge1, highp vec3 shiftEdge0, highp vec3 shiftEdge1)
{
    highp float smoothedHue = smoothstep(hueEdge0, hueEdge1, hsv.x);
    highp float hue = hsv.x + (shiftEdge0.x + ((shiftEdge1.x - shiftEdge0.x) * smoothedHue));
    highp float sat = hsv.y * (shiftEdge0.y + ((shiftEdge1.y - shiftEdge0.y) * smoothedHue));
    highp float lum = hsv.z * (shiftEdge0.z + ((shiftEdge1.z - shiftEdge0.z) * smoothedHue));
    return vec3(hue, sat, lum);
}

highp vec3 shift1 (highp vec3 hsl,lowp float pHue, lowp float nHue, highp vec3 pShift, highp vec3 shift,highp vec3 nShift, highp float whiteDistance)
{
    lowp float gradientOffset = 0.03;
    lowp float hue = mix(pHue,nHue,0.5);
    lowp float pDistance = 1.0 - smoothstep(pHue - gradientOffset, pHue + gradientOffset, hsl.x);
    lowp float nDistance = 1.0 - smoothstep(nHue + gradientOffset, nHue - gradientOffset, hsl.x);
    highp float h = 0.0;
    highp float s = 0.0;
    highp float l = 0.0;
    highp float bright = 1.0 - whiteDistance;
    highp float lShift = (1.0 - shift.z) * - (1.0-bright) + 1.0;
    highp float lpShift = (1.0 - pShift.z) * - (1.0-bright) + 1.0;
    highp float lnShift = (1.0 - nShift.z) * - (1.0-bright) + 1.0;
    if(hsl.x <= hue)
    {
        h = hsl.x + (shift.x + ((pShift.x - shift.x) * pDistance));
        s = hsl.y * (shift.y + ((pShift.y - shift.y) * pDistance));
        l = hsl.z * (lShift + ((lpShift - lShift) * pDistance));
    }
    else
    {
        h = hsl.x + (shift.x + ((nShift.x - shift.x) * nDistance));
        s = hsl.y * (shift.y + ((nShift.y - shift.y) * nDistance));
        l = hsl.z * (lShift + ((lnShift - lShift) * nDistance));
    }
    return vec3(h,s,l);
}

void main()
{
    //lowp vec3 redShift = vec3(h * 0.05, s * 0.6 + 1., l * 0.35 + 1.);
    lowp float red = 0.0;
    lowp float orange = mix(redOrange, orangeYellow, 0.5);
    lowp float yellow = mix(orangeYellow, yellowGreen, 0.5);
    lowp float green = mix(yellowGreen, greenAqua, 0.5);
    lowp float aqua = mix(greenAqua, aquaBlue, 0.5);
    lowp float blue = mix(aquaBlue, bluePurple, 0.5);
    lowp float purple = mix(bluePurple, purlpeMagenta, 0.5);
    lowp float magenta = mix(purlpeMagenta, magentaRed, 0.5);
    lowp vec4 color = texture2D(inputImageTexture0, texcoordOut);
    highp vec3 hsl = RGBtoHSL(color.rgb);
    highp vec3 lab = RGB2Lab(color.rgb);
    highp float whiteDistance = distance(lab,labWhiteColor) / 100.0;
    if (hsl.x < redOrange){
        hsl = shift1(hsl,0.0,redOrange,magentaShift,redShift,orangeShift,whiteDistance);
    }
    else if (hsl.x >= redOrange && hsl.x < orangeYellow) {
        hsl = shift1(hsl,redOrange,orangeYellow,redShift,orangeShift,yellowShift,whiteDistance);
    }
    else if (hsl.x >= orangeYellow && hsl.x < yellowGreen) {
        hsl = shift1(hsl,orangeYellow,yellowGreen,orangeShift,yellowShift,greenShift,whiteDistance);
    }
    else if (hsl.x >= yellowGreen && hsl.x < greenAqua) {
        hsl = shift1(hsl,yellowGreen,greenAqua,yellowShift,greenShift,aquaShift,whiteDistance);
    }
    else if (hsl.x >= greenAqua && hsl.x < aquaBlue) {
        hsl = shift1(hsl,greenAqua,aquaBlue,greenShift,aquaShift,blueShift,whiteDistance);
    }
    else if (hsl.x >= aquaBlue && hsl.x < bluePurple) {
        hsl = shift1(hsl,aquaBlue,bluePurple,aquaShift,blueShift,purpleShift,whiteDistance);
    }
    else if (hsl.x >= bluePurple && hsl.x < purlpeMagenta) {
        hsl = shift1(hsl,bluePurple,purlpeMagenta,blueShift,purpleShift,magentaShift,whiteDistance);
    }
    else if (hsl.x >= purlpeMagenta && hsl.x < magentaRed) {
        hsl = shift1(hsl,purlpeMagenta,magentaRed,purpleShift,magentaShift,redShift,whiteDistance);
    }
    else if (hsl.x >= magentaRed){
        hsl = shift1(hsl,magentaRed,1.0,magentaShift,redShift,orangeShift,whiteDistance);
    }
    highp vec3 rgb = hsl2rgb(hsl);
    gl_FragColor = vec4(rgb,color.a);
}

